/*******************************************************************************
**
** Copyright 2013 Sony Corporation
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License, version 2.1, as published by the Free Software Foundation.
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
**
*******************************************************************************/

#ifndef APPINTERFACE_H
#define APPINTERFACE_H

class QTmRemoteServer;
class QTmApplication;

/**
This is an abstract class(Interface) that is expected to be extended by every class that is to accept response from the Library.
This class is a interface to the TM Qt library Callbacks.
*/
class AppInterface
{
public:
    enum TmRequestStatus {
        SUCCESS                         = 0,
        FAILURE                         = -1,
        ERR_INVALID_APP_ID              = -2,
        ERR_FB_CREATION_FAILED          = -3,
        ERR_INVALID_SERVER_ID           = -4,
        ERR_SET_CLIENT_PROFILE_FAILED   = -5

    };

    enum TmResponse{
        VNC_CLIENT_STARTED = 1000,
        VNC_CLIENT_CONNECTED = 1010,
        VNC_CLIENT_RESIZED= 1020,
        VNC_CLIENT_DISCONNECTED= 1030,
        VNC_CLIENT_DISPLAY_CONFIGURATION= 1040,
        VNC_CLIENT_EVENT_CONFIGURATION= 1050,
        VNC_CLIENT_KEYBOARD_TRIGGER= 1060,
        VNC_CLIENT_KEYEVENT_LIST= 1070,
        VNC_CLIENT_FB_ALTERNATIVE_TEXT= 1080,
        VNC_CLIENT_CONTEXT_INFORMATION= 1090,
        ON_STATUS = 1100,
        ON_PROFILE_UPDATE = 1110,
        /** UPNP */
        REMOTE_SERVER_DEVICE_ADDED= 1120,
        REMOTE_SERVER_DEVICE_REMOVED= 1130,
        /** FROM REMOTE SERVER */
        APPLICATION_LAUNCHED= 1140,
        APPLICATION_TERMINATED= 1150,
        APPLICATION_STATUS_CHANGED = 1160,
        APPLICATION_ADDED= 1170,
        APPLICATION_LISTING_CHANGED= 1180,
        /** TM SURFACE*/
        FRAME_BUFFER_ALLOCATED = 1190,
        FRAME_BUFFER_DEALLOCATED = 1200,
        UPDATE_FRAME_BUFFER = 1210,

        /** TM ORTP*/
        RTP_PLAY_STATUS_CHANGE = 1300,
    };

    /**
        Callback method called by library when LibInterface::vncStartClient() succeeds
        @param clientId client identifier as was provided by vncClientStarted(int clientId)
    */
    virtual void vncClientStarted(int clientId) = 0;
    /**
        Callback method called by library when LibInterface::vncConnectClient(char* appId, int clientId) succeeds
        @param clientId This is the identifier associated with this client
    */
    virtual void vncClientConnected(int clientId) = 0;
    /**
        Callback mthod called client side frame buffer has changed in dimensions
        @param width Width of new frame buffer size
        @param height Height of new frame buffer size
        @param clientId client identifier as was provided by vncClientStarted(int clientId)
    */
    virtual void vncClientResized(int width, int height, int clientId) = 0;
    /**
        Callback method called by library when LibInterface::vncDisconnectClient(int clientId) succeeds
        @param clientId client identifier as was provided by vncClientStarted(int clientId)
    */
    virtual void vncClientDisconnected(int clientId) = 0;
    /**
        Callback method invoked when TmServer sends a display configuration message
        @param clientId client identifier as was provided by vncClientStarted(int clientId)
    */
    virtual void vncClientDisplayConfiguration(int clientId) = 0;
    /**
      Callback method invoked when TmServer sends a event configuration message
      @param clientId client identifier as was provided by vncClientStarted(int clientId)
    */
    virtual void vncClientEventConfiguration(int clientId, int knobKey, int deviceKey, int mediaKey) = 0;
    /**
      Callback method invoked when there is a need of diplaying a virtual keyboard at client end.
      @param clientId client identifier as was provided by vncClientStarted(int clientId)
      @param x x coordinate of where keyboard should be displayed
      @param y y coordinate of where keyboard should be displayed
      @param remove whether virtual keyboard should be removed or displayed
    */
    virtual void vncClientKeyboardTrigger(int x, int y, bool remove, int clientId) = 0;
    /**
      Callback method invoked when TmServer sends a key event list message.
      @param list list of keyids contained in key event list
      @param counter total number of keyids present in list
      @param clientId client identifier as was provided by vncClientStarted(int clientId)
    */
    virtual void vncClientKeyEventList(QList<unsigned int> *list, int counter, int clientId) = 0;
    /**
      Callback invoked when server sends alternate text message
      @param clientId client identifier as was provided by vncClientStarted(int clientId)
      @param appId application identifier for the application foro which alternate text is to be displayed
      @param text The text to be displayed.
    */
    virtual void vncClientFbAlternativeText(int appId, char* text, int clientId) = 0;
    /**
      Callback invoked when TmServer sends a context information for an application
      @param clientId client identifier as was provided by vncClientStarted(int clientId)
      @param x x coordinate of the application
      @param y y coordinate of the application
      @param width width of the application
      @param height height of the application
      @param appid application identifier for the application
      @param appCategory application category for the application
      @param appTrust trust level for this application
      @param displayCategory display category for this application
      @param displayRule display rule for this application
      @param displayTrust display trust level for this applictaion
    */
    virtual void vncClientContextInformation(int x, int y, int width, int height , int appId, unsigned int appCategory, int appTrust, int displayCategory, int displayRule, int displayTrust, int clientId) = 0;

    /**
      Callback invoked for all methods for which there are no explicite callback method or incase a particular LibInterface call fails.
      @param request identifier for the call being made LibInterface::TmRequest
      @param status status of the call

    */
    virtual void onStatus(LibInterface::TmRequest request,TmRequestStatus status) = 0;
    /**
      Callback invoked profile configured at the TmServer side is updated for agiven profileID.
      onProfileUpdate() can be invoked when LibInterface::verifyTmClientProfile(QTmClientProfile *clientProfile, int profileId, int serverId) is called or
      when LibInterface::setTmClientProfile(QTmClientProfile *clientProfile, int profileId, int serverId) is called;
      @param request identifier for the call being made LibInterface::TmRequest
      @param status status of the call
    */
    virtual void onProfileUpdate(LibInterface::TmRequest request, QTmClientProfile* profile, int profileID) = 0;


    /**
        This is the callback that is invoked when LibInterface::upnpConnectControlPoint(char* interfaceName, char* address) succeeds and a remote TmServer has been discovered.
        @param remoteServerDevice Value object representing a TmServer
    */
    virtual void remoteServerDeviceAdded(QTmRemoteServer *remoteServerDevice) = 0;
    /**
        This is the callback that is invoked when LibInterface::upnpDisconnectControlPoint(char* hostAddress) succeeds and a remote TmServer has been disconnected.
        @param serverId Identifier for the TmServer that has been disconnected.
    */
    virtual void remoteServerDeviceRemoved(int serverId) = 0;

    /**
        This is the callback invoked when LibInterface::upnpLaunchApplication(char* appId) succeeds.
        @param applicationId this is the application identifier for the application that is launched.
    */
    virtual void applicationLaunched(char* applicationId, char* url) = 0;
    /**
        This is the callback invoked when LibInterface::upnpTerminateApplication(char* appId) succeeds.
        @param applicationId this is the application identifier for the application that is launched.
    */
    virtual void applicationTerminated(char* applicationId) = 0;
    /**
        This is the callback invoked when LibInterface::upnpGetApplicationStatus(char* appId) succeeds or TmServer sends a app status change message.
        @param appId this is the application identifier for the application that is launched.
        @param status this is the current status for the application.
    */
    virtual void applicationStatusChanged(char* appId, QTmGlobal::TmStatus status) = 0;
    /**
        This is a callback that is invoked when TmServer is sends an application information.
        @param application Value object representing an application
    */
    virtual void applicationAdded(QTmApplication *application) = 0;
    /**
        This is a callback that is invoked when TmServer is sends multiple application information.
        @param applications list of application objects sent by the server.
    */
    virtual void applicationListingChanged(QList<QTmApplication*>*  applications) = 0;

    /**
        This is the callback called immediately after vncClientConnected(int clientId).
        This callback provides an application oppotunity to create a new bitmap object.
        Although a new frame buffer is allocated every time there is a change is frame buffer size in the library.

        @param buffer The underlying native buffer.
        @param size Total size of the buffer
        @param width width of the frame buffer
        @param height height of the frame buffer
        @param colorFormat The native colorformat being used
        @param clientId client identifier as was provided by vncClientStarted(int clientId)
    */
    virtual void frameBufferAllocated(unsigned char* buffer, int size,int width, int height, QTmGlobal::TmColorFormat  colorFormat, int clientId, double xScalingFactor, double yScalingFactor) = 0;
    /**
        This is the callback called before vncClientDisconnected().
        This callback provides an application oppotunity to delete a bitmap object.

        @param clientId client identifier as was provided by vncClientStarted(int clientId)
    */
    virtual void frameBufferDeallocated(int clientId) = 0;
    /**
        This is the callback called when ever the frame buffer is updated.
        @param x x coordinate of the update region.
        @param y y coordinate of the update region.
        @param width width of the update region.
        @param height height of the update region.
        @param clientId client identifier as was provided by vncClientStarted(int clientId)
    */
    virtual void updateFrameBuffer(int x, int y, int width, int height, int clientId, int bufferId, unsigned char* buffer) = 0;
    /** This is callback function called whenever the play status of rtp audio changes.
        @param updated play status of rtp audio.
     */
    virtual void rtpPlayStatusChanged(int rtpPlayStatus) = 0;
};

#endif // APPINTERFACE_H
