/*******************************************************************************
**
** Copyright 2013 Sony Corporation
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License, version 2.1, as published by the Free Software Foundation.
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
**
*******************************************************************************/

#ifndef LIBINTERFACE_H
#define LIBINTERFACE_H

class QTmClientProfile;
/**
This is an abstract class(Interface) that is expected to be extended by every class that exposes functionality provided by the interface.
This class is a interface to the TM Qt library APIs.
*/
class LibInterface
{

public:
    /** Start upnp control point
        @return 0 on success else error code will be returned
    */
    virtual int upnpStartControlPoint() = 0;
    /** Stop upnp control point. This call should only be made after successfull upnpStartControlPoint() call.*/
    virtual int upnpStopControlPoint() = 0;
    /**
        connect upnp control point. This call should only be made after successfull upnpStartControlPoint() call.
        @param interfaceName This is the name of the interface(eth1,usb0 etc.)
        @param address This is the IP address of the interfaceName
        @return 0 on success else error code will be returned
    */
    virtual int upnpConnectControlPoint(char* interfaceName, char* address) = 0;
    /**
        disconnect an already connected upnp control point. This call should only be made after successfull upnpConnectControlPoint() call.
        @param hostAddress This is the IP address that is being used a upnp control point
        @return 0 on success else error code will be returned
    */
    virtual int upnpDisconnectControlPoint(char* hostAddress) = 0;
    /**
        Obtain application status for a given application identified by appId.
        This call should only be made after successfull upnpConnectControlPoint() call.
        @param appId This is the application identifier for the application whose status is being sought.
        @return 0 on success else error code will be returned
    */
    virtual int upnpGetApplicationStatus(char*  appId) = 0;
    /**
        Launch application for a given application identified by appId.
        This call should only be made after successfull upnpConnectControlPoint() call.
        @param appId This is the application identifier for the application whose status is being sought.
        @return 0 on success else error code will be returned
    */
    virtual int upnpLaunchApplication(char* appId) = 0;//QTmApplication *application);
    /**
        Terminate application identified by appId.
        This call should only be made after successfull upnpConnectControlPoint() and upnpLaunchApplication() call.
        @param appId This is the application identifier for the application whose status is being sought.
        @return 0 on success else error code will be returned
    */
    virtual int upnpTerminateApplication(char* appId) = 0;
    /**
        Verify client profile against a profile identified by profileId configured to a Tm Server identified by serverId
        This call should only be made after successfull upnpConnectControlPoint() call.
        @param clientProfile This is the client profile that is being verified.
        @param profileId profile identifier against which the clientProfile is to be verified
        @param serverId server identifier for a given Tm Server
        @return 0 on success else error code will be returned
    */
    virtual int verifyTmClientProfile(QTmClientProfile *clientProfile, int profileId, int serverId) = 0;
    /**
        Obtain client profile identified by profileId configured to a Tm Server identified by serverId
        This call should only be made after successfull upnpConnectControlPoint() call.
        @param profileId profile identifier against which the clientProfile is to be verified
        @param serverId server identifier for a given Tm Server
        @return 0 on success else error code will be returned
    */
    virtual int getTmClientProfile(int profileId, int serverId) = 0;
    /**
        This is used to reconfigure a client profile identified by profileId to a Tm Server identified by serverId.
        This call should only be made after successfull upnpConnectControlPoint() call.
        @param clientProfile This is the client profile that is being configured.
        @param profileId profile identifier against which the clientProfile is to be verified
        @param serverId server identifier for a given Tm Server
        @return 0 on success else error code will be returned
    */
    virtual int setTmClientProfile(QTmClientProfile *clientProfile, int profileId, int serverId) = 0;

    /**
        Create an instance of RfbClient.
        This call should only be made after successfull upnpConnectControlPoint() and upnpLaunchApplication() call.
        @return client identifier on success else error code will be returned
    */
    virtual int vncStartClient() = 0;
    /**
        This is used to connect to a RFB server with application identified by appid as client identified by clientId.
        This call should only be made after successful vncStartClient() has been made.
        @param appId This is the application identifier that is that is to be launched.
        @param clientId This is the client identifier that was returned in vncStartClient() call.

        @return 0 on success else error code will be returned
    */
    virtual int vncConnectClient(char* appId, int clientId) = 0;
    /**
        This is used to disconnect from a RFB server as client identified by clientId.
        This call should be made only after successfull vncConnectClient() has been made.
        @param clientId This is the client identifier that was returned in vncStartClient() call.

        @return 0 on success else error code will be returned
    */
    virtual int vncDisconnectClient(int clientId) = 0;
    /**
        This is used to delete the reference of RFBClient that was created as a result of vncStartClient() call.
        This call should only be after successful vncDisconnectClient() call.
        @param appId This is the application identifier that is that is to be launched.
        @param clientId This is the client identifier that was returned in vncStartClient() call.

        @return 0 on success else error code will be returned
    */
    virtual int vncStopClient(int clientId) = 0;
    /**
        This is used to set the color format to be used frame buffer.
        @param colorFormat This is the color format to be used.
        @param clientId This is the client identifier that was returned in vncStartClient() call.

        @return 0 on success else error code will be returned
    */
    virtual int vncSetColorFormat(int colorFormat, int clientId) = 0;
    /**
        This is used to enable or diable RLE encoding.
        @param enable This is used enable or diable RLE encoding.
        @param clientId This is the client identifier that was returned in vncStartClient() call.

        @return 0 on success else error code will be returned
    */
    virtual int vncSetRunLengthEncoding(bool enable, int clientId) = 0;
    /**
        This is used to enable or diable incremental updates.
        @param enable This is used enable or diable incremental updates.
        @param clientId This is the client identifier that was returned in vncStartClient() call.

        @return 0 on success else error code will be returned
    */
    virtual int vncSetIncrementalUpdate(bool enable, int clientId) = 0;
    /**
        This is used to set preffered size of the frame buffer.
        @param width Width of frame buffer.
        @param height Height of frame buffer.
        @param scaling enable or diable client side scaling.
        @param clientId This is the client identifier that was returned in vncStartClient() call.

        @return 0 on success else error code will be returned
    */
    virtual int vncSetPreferredSize(int width, int height, bool scaling, int clientid, double clientSinglePixelWidth, double clientSinglePixelHeight, bool considerPixelComp) = 0;
    /**
        This is used to communicate virtual keyboard event to TMServer.
        @param keySym This is the key identifier of the virtual key that has been pressed.
        @param clientId This is the client identifier that was returned in vncStartClient() call.

        @return 0 on success else error code will be returned
    */

    virtual int vncOnFBReadComplete(int bufferId)=0;

    virtual int vncClickVirtualKey(int clientId, int keySym) = 0;
    /**
        This is used to communicate pointer/touch events to TMServer.
        @param x This is x coordinate of the point touched.
        @param y This is y coordinate of the point touched.
        @param pressed This is states the pressed status of the point touched.
        @param clientId This is the client identifier that was returned in vncStartClient() call.

        @return 0 on success else error code will be returned
    */
    virtual int vncPointerEvent(int clientId, int x, int y, unsigned int pressed) = 0;
    /**
        This is used to communicate phone hard keys event to TMServer.
        @param keyId This is the key identifier of the key that has been pressed.
        @param clientId This is the client identifier that was returned in vncStartClient() call.
        @param pressed This is states the pressed status of the point touched.

        @return 0 on success else error code will be returned
    */
    virtual int vncKeyEvent(int clientId, int keyId, bool pressed) = 0;
    /**
        This is used to block an application specific framebuffer.
        @param appId This is the application identifier that is that is to be blocked.
        @param x This is x coordinate of the top left rectangle to be blocked.
        @param y This is y coordinate of the top left rectangle to be blocked.
        @param width This is the width of the rectangle to be blocked.
        @param height This is the height of the rectangle to be blocked.
        @param reason This is reason for which the frame is to be blocked.
        @param id This is the client identifier that was returned in vncStartClient() call.
        @return 0 on success else error code will be returned
    */
    virtual int vncBlockFramebuffer(int x, int y, int width, int height, int appId, int reason, int id) = 0;

    /**
        This is used to reset the framebuffer update request timer.
        @return 0 on success else error code will be returned
    */
    virtual int vncResetFbUpdateRequestTimer() = 0;

    /**
        This is used to pause the sending of framebuffer update request.
        @return 0 on success else error code will be returned
    */
    virtual int vncPauseFbUpdateRequest() = 0;
    /**
        This is used to resume the sending of framebuffer update request.
        @return 0 on success else error code will be returned
    */
    virtual int vncResumeFbUpdateRequest() = 0;

    /**
        This is used to inform TM server about the client supported key events
        @return 0 on success else error code will be returned
    */
    virtual int vncSupportedKeyEvents(int clientId, int knobKey, int deviceKey, int mediaKey) = 0;

    enum TmRequest {
        UPNP_START_CONTROL_POINT		= 10,
        UPNP_CONNECT_CONTROL_POINT		= 20,
        TM_VERIFY_CLIENT_PROFILE		= 30,
        TM_GET_CLIENT_PROFILE			= 40,
        TM_SET_CLIENT_PROFILE			= 50,
        UPNP_DISCONNECT_CONTROL_POINT           = 60,
        UPNP_STOP_CONTROL_POINT			= 70,
        UPNP_GET_APPLICATION_STATUS		= 80,
        UPNP_LAUNCH_APPLICATION			= 90,
        UPNP_TERMINATE_APPLICATION		= 100,

        VNC_START_CLIENT			= 500,
        VNC_CONNECT_CLIENT			= 510,
        VNC_DISCONNECT_CLIENT			= 520,
        VNC_STOP_CLIENT				= 530,
        VNC_SET_COLOR_FORMAT			= 540,
        VNC_SET_RUNLENGTH_ENCODING		= 550,
        VNC_SET_INCREMENTAL_UPDATE		= 560,
        VNC_SET_PREFERRED_SIZE			= 570,
        VNC_CLICK_VIRTUAL_KEY			= 580,
        VNC_POINTER_EVENT			= 590,
        VNC_KEY_EVENT				= 600,
        VNC_BLOCK_FRAME_BUFFER			= 610,
        VNC_ON_FB_READ_COMPLETE			= 710,
        RTP_START_CLIENT                        = 611,
        RTP_CONNECT_CLIENT                      = 612,
        RTP_DISCONNECT_CLIENT                   = 613,
        RTP_STOP_CLIENT                         = 614,
        RTP_RESET_CLIENT			= 615,
        VNC_RESET_FBUPDATE_REQ_TIMER	= 616,
        VNC_PAUSE_FBUPDATE_REQ  		= 617,
        VNC_RESUME_FBUPDATE_REQ  		= 618,
        VNC_SUPPORTED_KEY_EVENTS  		= 619,
        INTERNAL                                = 1000
    };




};

#endif // LIBINTERFACE_H
