/*******************************************************************************
**
** Copyright 2013 Sony Corporation
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License, version 2.1, as published by the Free Software Foundation.
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
**
*******************************************************************************/
/*
 * ChangeLog -  
 *				March 19, 2013
 * 				1. Added suooprt for integrating the double buffer implementation.
 */

#include "QTmConfiguration.h"
#include <FrameBuffer.h>
#include <qtmsurfaceimpl.h>
#include <libinterface.h>
#include <appinterface.h>
#include <qsize.h>
#include <qrect.h>
#include <qdebug.h>
#include <QtCore/qmutex.h>
#include <QtCore/QWaitCondition>
#include <sys/timeb.h>
#include <QThread>


#define BUFFER_ID_START     128
#define MAX_BUFFER_COUNT    2

#define ENABLE_FBMANAGER_TIME_PROFILE

#ifdef ENABLE_FBMANAGER_TIME_PROFILE
//{
#define LOG_READ_COMPLETE   "DOUBLE_BUFFER_LOG :  READ COMPLETE  : "
#define LOG_WRITE_COMPLETE  "DOUBLE_BUFFER_LOG : WRITE COMPLETE  : "
#define LOG_SEND_FB_REQ     "DOUBLE_BUFFER_LOG : SENDING FB REQ  : "

#define PRINT_TIME_MSEC(func, buffId) {qDebug()<<func<<"time msec :" << QString::number ( getCurrMillisec(), 'g', 15) << "bufferId:" << buffId;}
#else
#define PRINT_TIME_MSEC(func, buffId)
//}
#endif

QTmSurfaceImpl* QTmSurfaceImpl::m_instance = 0;

QTmSurfaceImpl* QTmSurfaceImpl::getInstance()
{
    if(0 == m_instance)
    {
        m_instance = new QTmSurfaceImpl();
    }
    return m_instance;
}

QTmSurfaceImpl::QTmSurfaceImpl():m_clientId(-1),m_framebuffer(NULL)/*,framebuffer_ptrFlag(false), m_localBuffer(0), m_bufferSize(0)*/
{    
    mBuffCount = MAX_BUFFER_COUNT;

    mAwaitingFBUpdateBMState = new AwaitingFBUpdateBMState(this);
    mIdleBMState             = new IdleBMState(this);

    int bufferId = BUFFER_ID_START;
    for(int i=0; i < mBuffCount; i++)
    {
            FrameBuffer *fb = new FrameBuffer(this, bufferId++);
            mBuffersArray.append(fb);
    }
    // initially all the buffers will be in idle state.
    mCurrMLBMgrState = mIdleBMState;

    m_allocated         = false;
}

QTmSurfaceImpl::~QTmSurfaceImpl()
{
#ifdef QT_CODE_DEBUG
    qDebug()<<"QTmSurfaceImpl::~QTmSurfaceImpl()";
#endif

#ifdef BUFFER_MANAGER_INTEGRATED
    delete mIdleBMState;
    delete mAwaitingFBUpdateBMState;

    QVector<FrameBuffer*>::Iterator it;
    FrameBuffer* fb = NULL;
    for(it = mBuffersArray.begin(); it!=mBuffersArray.end(); it++){
        fb = (*it);
        delete fb;
    }

    mCurrMLBMgrState    = NULL;

    if(m_framebuffer)
    {
        free(m_framebuffer);
    }

    m_allocated         = false;
#else
    deleteFrameBuffer();
#endif
}

#ifdef BUFFER_MANAGER_INTEGRATED
void QTmSurfaceImpl::deleteFrameBuffer()
{
#ifdef QT_CODE_DEBUG
    qDebug()<<"QTmSurfaceImpl::deleteFrameBuffer()" << m_framebuffer;
#endif

    if(m_framebuffer)
    {
        free(m_framebuffer);
        m_framebuffer = NULL;
    }
}
#endif

int QTmSurfaceImpl::getClientId()
{
    return m_clientId;
}

void QTmSurfaceImpl::setClientId(int clientId)
{
    this->m_clientId = clientId;
}

uchar* QTmSurfaceImpl::allocateSurface(const QSize& s, QTmGlobal::TmColorFormat colorFormat, double xScalingFactor, double yScalingFactor)
{
    //qDebug()<<__PRETTY_FUNCTION__<<"DOUBLE_BUFFER_LOG : WAITING FOR MUTEX .................";

    int width, height;
    uchar * framebuffer_ptr;

    int pixel_width = 0;

        switch (colorFormat) {
        case QTmGlobal::RGB888:
            pixel_width = 4;
            break;
        case QTmGlobal::RGB565:
        case QTmGlobal::RGB555:
        case QTmGlobal::RGB444:
        case QTmGlobal::RGB343:
            pixel_width = 2;
            break;
        case QTmGlobal::Gray16:
        case QTmGlobal::Gray8:
        case QTmGlobal::RGBNone:;
        }

     int size = pixel_width * s.width() * s.height();

#ifdef BUFFER_MANAGER_INTEGRATED

     m_mutex.lock();
     qDebug()<<__PRETTY_FUNCTION__<<"DOUBLE_BUFFER_LOG : MUTEX LOCKED";

    if(m_allocated == false)
    {
        QVector<FrameBuffer*>::Iterator it;
        FrameBuffer* fb = NULL;

        for(it = mBuffersArray.begin(); it!=mBuffersArray.end(); it++){
            fb = (*it);
            fb->allocateBuffer((pixel_width * (QTmConfiguration::ConfigMaxDisplayWidth * QTmConfiguration::ConfigMaxDisplayHeight))+1);
        }

        FrameBuffer * ib = getFrameBufferByState(FB_STATE_IDLE);
        if(ib){
            framebuffer_ptr = ib->getDataPtr();

            m_allocated = true;
            //m_framebuffer = framebuffer_ptr;
            int size = (pixel_width * (QTmConfiguration::ConfigMaxDisplayWidth * QTmConfiguration::ConfigMaxDisplayHeight))+1;
            m_framebuffer = new unsigned char[size];
            memset(m_framebuffer, 0, size);
            m_bufferSize = size;
        }
        else
        {
             qDebug()<<__PRETTY_FUNCTION__<<"DOUBLE_BUFFER_LOG : IDLE BUFFER NOT AVAILABLE";
            
        }

    }
    else
    {
        //this means fb is already  allocated
        //this happens when desktop size changes

        //TODO: may be current writing buffer needs to be emptied !!
    }

     m_mutex.unlock();
     qDebug()<<__PRETTY_FUNCTION__<<"DOUBLE_BUFFER_LOG : MUTEX UNLOCKED";

#else

     if(!m_framebuffer)
     {
         m_framebuffer = (unsigned char *) malloc((pixel_width * (QTmConfiguration::ConfigMaxDisplayWidth * QTmConfiguration::ConfigMaxDisplayHeight))+1);
     }

     memset(m_framebuffer, 0, (pixel_width * QTmConfiguration::ConfigMaxDisplayWidth * QTmConfiguration::ConfigMaxDisplayHeight)+1);

#endif

     if((s.width() <= QTmConfiguration::ConfigMaxDisplayWidth) && (s.height() <= QTmConfiguration::ConfigMaxDisplayHeight))
     {
         width = s.width();
         height = s.height();
     }
     else
     {
         width = QTmConfiguration::ConfigMaxDisplayWidth;
         height = QTmConfiguration::ConfigMaxDisplayHeight;
     }

    //'m_framebuffer' passed in this callback should not be used anywhere in app layer
     m_appInterface->frameBufferAllocated(m_framebuffer,size + 1,width,height,colorFormat, m_clientId, xScalingFactor, yScalingFactor);

    return m_framebuffer;
}

void QTmSurfaceImpl::deallocateSurface()
{
    m_appInterface->frameBufferDeallocated(m_clientId);
}

void QTmSurfaceImpl::updateSurface(const QRect& updateRect)
{    
    m_mutex.lock();
    qDebug()<<__PRETTY_FUNCTION__<<"DOUBLE_BUFFER_LOG : MUTEX LOCKED";

    struct timeb currTime;
    ftime(&currTime);
    double timeMilli = (1000.0*currTime.time  + currTime.millitm);

    FrameBuffer * writingBuffer = getFrameBufferByState(FB_STATE_WRITING);

    if(NULL != writingBuffer){

    //copy data into 'writingBuffer' from 'm_framebuffer'
    memcpy(writingBuffer->getDataPtr(), m_framebuffer, m_bufferSize);

//#ifdef QT_CODE_DEBUG
    PRINT_TIME_MSEC(LOG_WRITE_COMPLETE, writingBuffer->getId())
//#endif
        qDebug()<<"DOUBLE_BUFFER_LOG : Changing state from AWAITING -> IDLE of BUFFER MANAGER";

        setCurrentState(mIdleBMState);

        writingBuffer->prepareForReading();

        int bufferId = writingBuffer->getId();
        unsigned char* bufferPtr = writingBuffer->getDataPtr();

        
        m_appInterface->updateFrameBuffer(updateRect.x(),updateRect.y(),updateRect.width(),updateRect.height(), m_clientId, bufferId, bufferPtr);
    }else{
        qDebug()<<"DOUBLE_BUFFER_LOG : IMPOSSIBLE!!! : No buffer is WRITING";
    }

    m_mutex.unlock();
    qDebug()<<__PRETTY_FUNCTION__<<"DOUBLE_BUFFER_LOG : MUTEX UNLOCKED";

}

void QTmSurfaceImpl::setAppInterface(AppInterface* appInterface)
{
#ifdef QT_CODE_DEBUG
    qDebug()<<"QTmSurfaceImpl::setAppInterface()" << appInterface;
#endif
    this->m_appInterface = appInterface;
}

/************************************************************************************************************
 ************************************************************************************************************
 ***********************************FRAME BUFFER MANAGER INTEGRATED******************************************
 ************************************************************************************************************
 ************************************************************************************************************/

void QTmSurfaceImpl::start(){

    m_mutex.lock();
    qDebug()<<__PRETTY_FUNCTION__<<"DOUBLE_BUFFER_LOG : MUTEX LOCKED";

    setCurrentState(mAwaitingFBUpdateBMState);
    startWritingToIdleBuffer();

    m_mutex.unlock();
    qDebug()<<__PRETTY_FUNCTION__<<"DOUBLE_BUFFER_LOG : MUTEX UNLOCKED";
}
void QTmSurfaceImpl::onBufferReadComplete(int bufferId)
{
    m_mutex.lock();
    qDebug()<<__PRETTY_FUNCTION__<<"DOUBLE_BUFFER_LOG : MUTEX LOCKED";

    PRINT_TIME_MSEC(LOG_READ_COMPLETE, bufferId)

    FrameBuffer* readingBuffer = getFrameBufferById(bufferId);
    if(readingBuffer){
        readingBuffer->prepareForIdle();
    }else{
        qDebug()<<__PRETTY_FUNCTION__<< "DOUBLE_BUFFER_LOG : IMPOSSIBLE!!! : reading buffer received is null";
    }

    m_mutex.unlock();
    qDebug()<<__PRETTY_FUNCTION__<<"DOUBLE_BUFFER_LOG : MUTEX UNLOCKED";

}

FrameBuffer* QTmSurfaceImpl::getFrameBufferById(int bufferId){
    FrameBuffer* currBuff;
    QVector<FrameBuffer*>::iterator it;
    for(it = mBuffersArray.begin(); it!=mBuffersArray.end();it++){
        currBuff = (*it);
        if(bufferId == currBuff->getId()){
            return currBuff;
        }
    }
    return NULL;
}

/*************************** state machine methods ***************************/
FrameBufferManagerState* QTmSurfaceImpl::getIdleBMState(){
    return mIdleBMState;
}

FrameBufferManagerState* QTmSurfaceImpl::getAwaitingFBUpdateBMState(){
    return mAwaitingFBUpdateBMState;
}

void QTmSurfaceImpl::setCurrentState(FrameBufferManagerState* state){
    mCurrMLBMgrState = state;
}

void QTmSurfaceImpl::onBufferAbnormalStateChange(FrameBuffer buffer)
{
    mCurrMLBMgrState->onBufferAbnormalStateChange();
}

void QTmSurfaceImpl::onBufferNormalStateChange(){

    //if FB manager is IDLE then only we need to notify RFBClient for sending the FB Update request
    mCurrMLBMgrState->onBufferNormalStateChange();
}

void QTmSurfaceImpl::startWritingToIdleBuffer(){

    FrameBuffer* buff = NULL;

    buff = getFrameBufferByState(FB_STATE_IDLE);
    if(buff != NULL)
    {
        buff->prepareForWriting();
        PRINT_TIME_MSEC(LOG_SEND_FB_REQ, buff->getId())
        notify();
    }
}

/*************************** state machine methods ***************************/

FrameBuffer* QTmSurfaceImpl::getFrameBufferByState(FB_STATE buffState){
        FrameBuffer* currBuff = NULL;
        FB_STATE currState = FB_STATE_IDLE;
        QVector<FrameBuffer*>::iterator it;
        if(buffState < FB_STATE_MAX)
        {
            for(it = mBuffersArray.begin(); it!=mBuffersArray.end();it++)
            {
                if((*it) != NULL)
                {
                    currState = (*it)->getBufferStateId();

                    if(currState == buffState)
                    {
                        currBuff = (*it);
                        break;
                    }
                }
                else
                {
                    qDebug() << "DOUBLE_BUFFER_LOG : currBuff is null: ";
                }
            }
        }
        else
        {
            qDebug() << "DOUBLE_BUFFER_LOG : invalid buffer state requested !! ";
        }

        return currBuff;
}


void QTmSurfaceImpl::notify(){
    qDebug()<<"DOUBLE_BUFFER_LOG : N.O.T.I.F.Y";
    QVector<SurfaceObserver *>::Iterator it;
    for(it = mObserverArray.begin();it!=mObserverArray.end();it++){

        SurfaceObserver *so = (*it);
        if(so!=NULL){
           so->updateIdleBufferAvailability();
       }else{
           qDebug() << "DOUBLE_BUFFER_LOG : so is null";
       }
    }
}

unsigned char*  QTmSurfaceImpl::getSurface(){

    //this pointer is needed by RFBClient to fill the data received from socket
    //since at a time only one frame is received, there will be only one FrameBuffer in WRITING state
    unsigned char* ptr = NULL;
    FrameBuffer * buff = NULL;
    buff = getFrameBufferByState(FB_STATE_WRITING);
    if(buff != NULL)
    {
        qDebug()<<"DOUBLE_BUFFER_LOG : Sending fbupdaterequest for buffer : "<< buff->getId();
        ptr = buff->getDataPtr();
    }

    //always write into scratchpad buffer :)
    return m_framebuffer;

}


void QTmSurfaceImpl::resetSurface(){
	m_mutex.lock();
	qDebug()<<__PRETTY_FUNCTION__<<"DOUBLE_BUFFER_LOG : MUTEX LOCKED";
    QVector<FrameBuffer*>::iterator it;
    FrameBuffer * currBuff = NULL;
    for(it = mBuffersArray.begin(); it!=mBuffersArray.end();it++){
        currBuff = (*it);
        if(currBuff!=NULL){
            qDebug() << "DOUBLE_BUFFER_LOG : resetting buffer : " <<  currBuff->getId();
            currBuff->reset();
        }else{
            qDebug() << "DOUBLE_BUFFER_LOG : currBuff is null: ";
        }
    }

    mCurrMLBMgrState = mIdleBMState;
    mObserverArray.clear();

    if(m_framebuffer)
    memset(m_framebuffer, 0, m_bufferSize);

    m_mutex.unlock();
    qDebug()<<__PRETTY_FUNCTION__<<"DOUBLE_BUFFER_LOG : MUTEX UNLOCKED";
}

void QTmSurfaceImpl::registerObserver(SurfaceObserver * observer){
    mObserverArray.append(observer);
}

double QTmSurfaceImpl::getCurrMillisec()
{
    struct timeb timeVal;
    double timeMsec;

    ftime(&timeVal);

    timeMsec = ((1000.0*timeVal.time) + timeVal.millitm);

    //qDebug() << "QTmSurfaceImpl::getCurrMillisec: curr msec" << QString::number(timeMsec, 'f', 0);
    return timeMsec;
}
